#!/usr/bin/perl -w
use strict;
use DBI;
use CGI qw(:standard);

#Ebbie: by H Alexander Ebhardt
#DISTRUBUTION UNDER GNU General Public License
#READ http://bioinformatics.org/ebbie/gnu.txt

require '/projects/home/aebhardt/ebbie/lib/blast.lib';
require '/projects/home/aebhardt/ebbie/lib/html.lib';
require '/projects/home/aebhardt/ebbie/lib/mysql.lib';
require '/projects/home/aebhardt/ebbie/lib/anal.lib';

my ( $query, $dna, @dna, $fh, $count, $filename, $sort, $id );
my ( $three, $rcthree, $five, $rcfive, $countins, $idsql );
my ( $insert, $rcinsert, $rcinsert2, @insert, @insertrc, $countid, $countall );
my ( $countempty, $pos, $len5, $pos5, $lenins, $len3, $pos3, $countpri );
my ( $countemptyrc, $posrc, $pos5rc, $leninsrc, $pos3rc, $countprirc );
my ( @array5, @array3, @array5rc, @array3rc );

my @blastdb = param('blastdb');
my $mysqldb = param('mysqldb');
$query    = new CGI;
$filename = $query->param('seqfile');
$filename =~ s/.*[\/\\](.*)/$1/;

#create web page
html_start();    #/lib/html

#if no file is selected, tell the user and provide option to go back to front page
$id = $filename;
if ( $id eq '' ) {
	print "<P>No file selected.<P>";
	frontp($mysqldb);    #/lib/html
	print "</BODY></HTML>";
	exit(0);
}

#if the seleted file has an identical entry in the MySQL database, tell the user
#provide option to go back to front page
$count = sqlcheck( $mysqldb, $id );    #/lib/mysql
if ( $count > 0 ) {

	#	print "<P><B>This id ($id) does already exist in the database.</B><P>";
	print
"<P><B>This file corresponding to $id has already been entered in the $mysqldb-database.</B></P>";
	frontp($mysqldb);                  #/lib/html
	print "</BODY></HTML>";
	exit(0);
}

#for logbook: first column: date, second column filename
open( FH, "date |" );
my $datelog = <FH>;
close(FH);
open( TEMP, ">>/projects/home/aebhardt/ebbie/log/$mysqldb" );
print TEMP "<tr><td>$datelog<td>$id\n";
close(TEMP);

#else, work on the file: convert text file into string
$fh  = $query->param('seqfile');
@dna = <$fh>;
$dna = join( '', @dna );
$dna =~ s/\s//g;
$dna =~ tr/acgt/ACGT/;

#retrieve 5' and 3' primers from database and make reverse complment of them
$five   = read5pr($mysqldb);
$len5   = length($five);
$rcfive = reverse $five;
$rcfive =~ tr/ACGT/TGCA/;
$three   = read3pr($mysqldb);
$len3    = length($three);
$rcthree = reverse $three;
$rcthree =~ tr/ACGT/TGCA/;

#search for sense inserts in $dna string deposits all inserts into @insert
$countempty = 0;
$pos        = 0;
while ( ( $pos = index( $dna, $five, $pos ) ) >= 0 ) {
	$pos5 = $pos + $len5;
	push( @array5, $pos5 );
	$pos3 = index( $dna, $three, $pos5 );
	push( @array3, $pos3 );
	$lenins = $pos3 - $pos5;
	if ( $lenins == 0 ) {
		$countempty++;
		$pos++;
	}
	else {
		push( @insert, substr( $dna, $pos5, $lenins ) );
		$pos++;
	}
}
print "<P>5is @array5, 3is @array3 sense<P>";

#/lib/anal.lib checks array if number of primers is even
checkarray( $mysqldb, @array5 );
checkarray( $mysqldb, @array3 );

#searches for all reverse complement inserts, deposits them into @insertrc
$countemptyrc = 0;
$posrc        = 0;
while ( ( $posrc = index( $dna, $rcthree, $posrc ) ) >= 0 ) {
	$pos3rc = $posrc + $len3;
	push( @array3rc, $pos3rc );
	$pos5rc = index( $dna, $rcfive, $pos3rc );
	push( @array5rc, $pos5rc );
	$leninsrc = $pos5rc - $pos3rc;
	if ( $leninsrc == 0 ) {
		$countemptyrc++;
		$posrc++;
	}
	else {
		push( @insertrc, substr( $dna, $pos3rc, $leninsrc ) );
		$posrc++;
	}
}

print "<P>3is @array3rc 5is @array5rc antisense<P>";
#/lib/anal.lib checks array if number of primers is even
checkarray( $mysqldb, @array3rc );
checkarray( $mysqldb, @array5rc );

#do some math: how many inserts, how many primer pairs without insert etc.
#this counting is important for the next big loop
my $ebbie2 = scalar @insert;
my $ebbie1 = scalar @insertrc;
my $ebbie  = $ebbie1 + $ebbie2;

#if there is no insert: no insert, sorry
#if there is 1 insert = analyze it
#if there are more than 1 inserts, deposit all inserts into database and print pull down menu
if ( $ebbie == 1 ) {

	# logfile entry:
	open( TEMP, ">>/projects/home/aebhardt/ebbie/log/$mysqldb" );
	print TEMP "<td>Single insert found.</tr>\n";
	close(TEMP);

	#/lib/anal deposit id, insert, length and source into MySQL db
	if ( $ebbie2 == 1 ) {
		foreach (@insert) {
			$insert = $_;
			depoins( $id, $insert, $mysqldb );    #lib/anal.lib
		}
	}
	if ( $ebbie1 == 1 ) {
		foreach (@insertrc) {
			$rcinsert  = $_;
			$rcinsert2 = reverse $rcinsert;
			$rcinsert2 =~ tr/ACGTN/TGCAN/;
			depoins( $id, $rcinsert2, $mysqldb );    #lib/anal.lib
		}
	}
}

#for each @insert / @insertrc deposit all inserts into database and then call them
elsif ( $ebbie > 1 ) {
	$countid = 1;
	foreach (@insert) {
		$insert = $_;
		$idsql  = $id . $countid;
		depoins( $idsql, $insert, $mysqldb );
		$countid++;
		$idsql = ();
	}
	foreach (@insertrc) {
		$rcinsert  = $_;
		$idsql     = $id . $countid;
		$rcinsert2 = reverse $rcinsert;
		$rcinsert2 =~ tr/ACGTN/TGCAN/;
		depoins( $idsql, $rcinsert2, $mysqldb );
		$countid++;
		$idsql = ();
	}

# call MySQL database and ask for all id's with no orientation, list these in a pulldown menu
	$countall = $countempty + $countemptyrc + $ebbie;
	table();
	print
"<HR>There were $countall primer pairs and $ebbie inserts deposited into $mysqldb.";

	# logfile entry:
	open( TEMP, ">>/projects/home/aebhardt/ebbie/log/$mysqldb" );
	print TEMP
"<td>There were $countall primer pairs and $ebbie inserts deposited into $mysqldb.</tr>\n";
	close(TEMP);

	print
	  "<FORM 
ACTION='http://bioinformatics.org/ebbie/cgi-bin/ebbie2.cgi' METHOD=post>";

	#/lib/mysql calls all entries with NULL orientation
	sqlebbie($mysqldb);
	print
"<BR><I>Please select one of the RNA inserts from the above pull down menu.</I>";

	#/lib/blast: provides checklist for BlastN searches
	blastcheck();

	#/lib/mysql: provides checklist for MySQL databases
	mysqldb($mysqldb);
	print "<P><INPUT TYPE='submit' VALUE='Ebbie - sequence analysis'></FORM>";
	table_end();
	print "</BODY></HTML>";
	exit(0);
}
else {
	print "<p><b>Sorry, there was no insert found.</b><p>";
	frontp($mysqldb);
	print "</BODY></HTML>";

	# logfile entry:
	open( TEMP, ">>/projects/home/aebhardt/ebbie/log/$mysqldb" );
	print TEMP "<td>Sorry, there was no insert found.</tr>\n";
	close(TEMP);

	exit(0);
}

#analysis common to all inserts
print
  qq(<form action="http://bioinformatics.org/ebbie/cgi-bin/annoch.cgi" 
method=post>\n);
print "<input type='hidden' name='id' value='$id' checked>";
$sort = $id;
my ( $anno, $countt ) = sqltable( $mysqldb, $sort, $id );    #/lib/mysql
mysqldb($mysqldb);                                           #/lib/mysql
annofields($mysqldb);                                        #/lib/html
print "<td><p><i>Orientation:</i>";
print <<MEM;
<select name="orientation">
<option value="not appl">N/A</option>
<option value="antisense">antisense</option>
<option value="sense">sense</option></select>
<p><input type="submit" value="Deposit annotations for '$id'."></form>
MEM
print
"<HR><H4>Review of BlastN searches (query: '$id') against local databases:<BR></H4>";

foreach (@blastdb) {
	blast( $_, $mysqldb );    #/lib/blast
}
print "</BODY></HTML>";

exit(0);
